#INCLUDE "\flash\codebook\common\include\framincl.h"

*==========================================================
DEFINE CLASS cAbstractFactory AS Custom
*==========================================================
*) Description:
*)	This object makes it possible to manufacture any object
*)	in the COMCodebook framework.  To do this, each object
*)	created requires certain environmental conditions.  For
*)	example, all required class libraries must be set, all 
*)	procedures must be set, and sometimes, a procedure or
*)	two must be run both before and after the successful
*)	manufacture of the object.  This class definition, 
*)	in collusion with the supporting table FACTORY.DBF, 
*)	enable the developer to accomplish these tasks.
*)	
*)	FACTORY.DBF
*)	1.	CKEY            Character    20  
*)		Label used to identify the object being created.
*)
*)	2.	CLIBRARY        Character    50  
*)		Specifies the name of the .VCX visual class library to open
*)		or, in the case of a procedure, the name of the .PRG to open. 
*)		
*)	3.	CCLASS          Character    50  
*)		The name of the class definition to be used to create
*)		the object.
*)
*)	4.	CFILE           Character    50  
*)		Specifies a Visual FoxPro application file (.app) or executable 
*)		file (.exe) containing the visual class library.
*)
*)	5.	MCLASSES        Memo          4  
*)		Comma delimited list of class libraries required to support 
*)		the object being manufactured.
*)	
*)	6.	MPROCS          Memo          4  
*)		Comma delimited list of procedure files required to support
*)		the object being manufactured.
*)
*)	7.	MPREPROC        Memo          4  
*)		Contains either lines of code to execute or a function call
*)		to an external program that should be executed before the
*)		object is manufactured.
*)		
*)    8  MPOSTPROC       Memo          4  
*)		Contains either lines of code to execute or a function call
*)		to an external program that should be executed after the
*)		object is manufactured.
*==========================================================

	lDisplayErrorMessage = .T.
	Name                 = "cAbstractFactory"
	cFactoryTable        = "FACTORY.DBF"
	

	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  Init()
	*-------------------------------------------------------
	PROCEDURE Init
		LPARAMETERS tcKey, roObject, tlSkipEnvironmentSetup
		LOCAL llRetVal
		
		roObject = .NULL.

		IF .NOT. EMPTY( tcKey ) AND TYPE( 'tcKey' ) == "C"
			llRetVal = THIS.ManufactureObject( tcKey, @roObject, tlSkipEnvironmentSetup )
		ELSE
			llRetVal = .T.
		ENDIF
		
		RETURN llRetVal
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Creates the factory and optionally, if the Key and
	*)	Reference Variable are provided, manufactures the 
	*)	specified object, returns a reference to it, then
	*)	prevents the factory itself from instantiating.
	*)	Creating the factory object with no parameters
	*)	results in a persistant factory object suitable for
	*)	repetitive use.
	*-------------------------------------------------------
	*$ Scope:       PUBLIC
	*$ Parameters:  
	*$	1.	cKey - the label used to lookup the tuple in the
	*$		FACTORY.DBF so the class and supporting information
	*$		can be extracted and used.
	*$	2.	oObject - variable, passed by reference, to where
	*$		the newly manufactured object is stored and subsequently
	*$		returned to the calling program.	
	*$ Usage:       
	*$	cAbstractFactory:Init( [<cKey>], [<@oObject>] )
	*$ Example:     
	*$	=CREATEOBJ( "AbstractFactory", "ApplicationObject", @loApp )
	*$ Returns:  LOGICAL
	*$	.T. - if the object is successfully created.
	*$	.F. - if the specified object is not created.
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////

	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  Release()
	*-------------------------------------------------------
	FUNCTION Release
		RELEASE This
	ENDFUNC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Enables the factory object to release itself.
	*-------------------------------------------------------
	*$ Scope:       PUBLIC
	*$ Parameters:  None
	*$ Usage:       
	*$	cAbstractFactory::Release()
	*$ Example:     
	*$	THIS.oFactory.Release()
	*$ Returns: LOGICAL .T. by default.
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////


	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  ManufactureObject()
	*-------------------------------------------------------
	PROCEDURE ManufactureObject
		LPARAMETERS tcKey, roObject, tlSkipEnvironmentSetup

		LOCAL loObject       , ;
		      lcClassName    , ;
		      lcClassLibrary , ;
		      lcClassFile    , ;
		      lcClassLibList , ;
		      lcProcList     , ;
		      lcPreProc      , ;
		      lcPostProc     , ;
		      llRetVal

		roObject = .NULL.		      
		IF tlSkipEnvironmentSetup
			lcClassName = tcKey
		ELSE
			This.GetClassInformation( tcKey          , ;
			                         @lcClassName    , ;
			                         @lcClassLibrary , ;
			                         @lcClassFile    , ;
			                         @lcClassLibList , ;
			                         @lcProcList     , ;
			                         @lcPreProc      , ;
			                         @lcPostProc       )
		ENDIF                         

		IF .NOT. tlSkipEnvironmentSetup
			This.SetClassLibraries( lcClassLibrary , ;
			                        lcClassFile    , ;
			                        lcClassLibList   )

			This.SetProcedures( lcProcList )

			This.ExecuteProcedure( lcPreProc )
		ENDIF
		
		roObject = This.InstantiateObject( lcClassName )

		IF .NOT. tlSkipEnvironmentSetup
			This.ExecuteProcedure( lcPostProc )
		ENDIF
		
		llRetVal = TYPE('roObject.Name') == "C"
		
		RETURN llRetVal
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Template method that defines the steps required 
	*)	to manufacture an object.
	*-------------------------------------------------------
	*$	1.	cKey - the label used to lookup the tuple in the
	*$		FACTORY.DBF so the class and supporting information
	*$		can be extracted and used.
	*$	2.	oObject - variable, passed by reference, to where
	*$		the newly manufactured object is stored and subsequently
	*$		returned to the calling program.	
	*$ Usage:       
	*$	cAbstractFactory:ManufactureObject( [<cKey>], [<@oObject>] )
	*$ Example:     
	*$	THIS.oFactory.ManufactureObject( "ApplicationObject", @loApp )
	*$ Returns:  LOGICAL
	*$	.T. - if the object is successfully created.
	*$	.F. - if the specified object is not created.
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////


	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  GetClassInformation()
	*-------------------------------------------------------
	PROTECTED PROCEDURE GetClassInformation
		LPARAMETERS tcKey          , ;
		            rcClassName    , ;
		            rcClassLibrary , ;
		            rcClassFile    , ;
		            rcClassLibList , ;
		            rcProcList     , ;
		            rcPreProc      , ;
		            rcPostProc

		LOCAL llOK2Close

		IF TYPE( 'tcKey' ) == "C" AND .NOT. EMPTY( tcKey )

			#DEFINE Class_Definition   1
			#DEFINE Class_Library      2
			#DEFINE Class_Library_File 3
			#DEFINE Class_Lib_List     4
			#DEFINE Proc_List          5
			#DEFINE Pre_Processing     6
			#DEFINE Post_Processing    7

			llOK2Close = .NOT. USED( 'factory' )

			SELECT cClass    , ;
			       cLibrary  , ;
			       cFile     , ;
			       mClasses  , ;
			       mProcs    , ;
			       mPreProc  , ;
			       mPostProc   ;
			  FROM ( THIS.cFactoryTable ) ;
			 WHERE ALLTRIM(UPPER(cKey)) == ALLTRIM(UPPER( tcKey )) ;
			  INTO ARRAY laFactory

			IF llOK2Close AND USED( 'factory' )
				USE IN factory
			ENDIF

			IF _TALLY = 1 
				rcClassName    = ALLTRIM(laFactory[ Class_Definition ])
				rcClassLibrary = ALLTRIM(laFactory[ Class_Library ])
				rcClassFile    = ALLTRIM(laFactory[ Class_Library_File ])
		        rcClassLibList = laFactory[ Class_Lib_List ]
		        rcProcList     = laFactory[ Proc_List ]
		        rcPreProc      = laFactory[ Pre_Processing ]
		        rcPostProc     = laFactory[ Post_Processing ]
			ELSE
				IF This.lDisplayErrorMessage
					=MsgSvc('CFACTORY::ClassLibraryNotFound', tcKey )
				ENDIF
			ENDIF

		ENDIF

		IF .NOT. EMPTY( rcClassName )
			RETURN .T.
		ELSE
			RETURN .F.
		ENDIF
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	This method is responsible for retrieving the object's
	*)	class information from the supporting factory table,
	*)	FACTORY.DBF by default.
	*-------------------------------------------------------
	*$ Scope:       PROTECTED
	*$ Parameters:  
	*$	1. 	<cKey> - key used to uniquely identify the 
	*$		required row in the supporting factory table.
	*$	2.	<cClassName> - name of the class definition
	*$		to use when creating the object.
	*$	3.	<cClassLibrary - name of the .VCX or .PRG
	*$		that contains the class definition.
	*$	4.	<cClassFile> - Specifies a Visual FoxPro 
	*$		application file (.app) or executable 
	*$		file (.exe) containing the visual class library.
	*$	5.	<cClassLibList> - comma delimited list of
	*$		class libraries needed to support the object
	*$		being manufactured
	*$	6.	<cProcList> - comma delimited list of 
	*$		procedures needed to support the object
	*$		being manufactured
	*$	7.	<cPreProc> - code or function/procedure
	*$		call that must be executed prior to creating
	*$		the object.
	*$	8.	<cPostProc> - code or function/procedure
	*$		call that must be executed after the 
	*$		successful creation of the object.
	*$ Usage:
	*$	CAbstractFactory::GetClassInformation( cKey           , ;
	*$										   @cClassName    , ;
	*$										   @cClassLibrary , ;
	*$										   @cClassFile    , ;
	*$										   @cClassLibList , ;
	*$										   @cProcList     , ;
	*$										   @cPreProc      , ;
	*$										   @cPostProc       )
	*$ Example:     
	*$	This.GetClassInformation( tcKey          , ;
	*$	                         @lcClassName    , ;
	*$	                         @lcClassLibrary , ;
	*$	                         @lcClassFile    , ;
	*$	                         @lcClassLibList , ;
	*$	                         @lcProcList     , ;
	*$	                         @lcPreProc      , ;
	*$	                         @lcPostProc       )
	*$ Returns:  LOGICAL
	*$	.T. if the class name is retrieved
	*$	.F. if it is not
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////


	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  ExecuteProcedure()
	*-------------------------------------------------------
	PROTECTED PROCEDURE ExecuteProcedure
		LPARAMETER tcProcedure
		LOCAL lcProgram
		
		IF .NOT. EMPTY( tcProcedure ) AND TYPE( 'tcProcedure' ) == "C"

			IF FILE( 'temp.prg' )
				ERASE temp.prg
			ENDIF

			IF FILE( 'temp.fxp')
				ERASE temp.fxp
			ENDIF

			IF STRTOFILE( tcProcedure, 'temp.prg' ) > 0
				lcProgram = "TEMP.PRG"
				DO (lcProgram)
			ENDIF

			IF FILE( 'temp.prg' )
				ERASE temp.prg
			ENDIF

			IF FILE( 'temp.fxp')
				ERASE temp.fxp
			ENDIF

		ENDIF
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Responsible for transferring the code to be executed
	*)	into a program file named, TEMP.PRG, and then executing
	*)	that program.  Both the .PRG and .FXP should not
	*)	exist after the program is run.
	*-------------------------------------------------------
	*$ Scope:       PROTECTED
	*$ Parameters:  
	*$	1.	<cProcedure> - contains the code that must be
	*$		transferred to a .PRG and executed.
	*$ Usage:       
	*$	CAbstractFactory::ExecuteProcedure( <cProcedure> )
	*$ Example:   
	*$	THIS.ExecuteProcedure( lcPreProc ) 
	*$	THIS.ExecuteProcedure( lcPostProc )
	*$ Returns:     LOGICAL .T. by default
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////


	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  SetClassLibraries()
	*-------------------------------------------------------
	PROTECTED PROCEDURE SetClassLibraries
		LPARAMETERS tcClassLibrary , ;
		            tcClassFile    , ;
		            tcClassLibList

		LOCAL lcClassList     , ;
		      lnTotalClasses  , ;
		      lnClassCount    , ;
		      lnEndPosition   , ;
		      lcClassLibrary  , ;
		      lnBeginPosition , ;
		      lcFileName
		            
		lcClassList = UPPER(ALLTRIM(SET( 'CLASSLIB' )))

		IF .NOT. EMPTY( tcClassLibList )

			lnTotalClasses = OCCURS( ",", tcClassLibList ) + 1

			FOR lnClassCount = 1 TO lnTotalClasses

				DO CASE
				CASE lnClassCount  = 1
					lnEndPosition  = AT( ",", tcClassLibList, 1 ) - 1
					lcClassLibrary = ALLTRIM(LEFT( tcClassLibList, lnEndPosition ))
				CASE lnClassCount = lnTotalClasses
					lnBeginPosition = AT( ",", tcClassLibList, lnClassCount-1 ) + 1
					lcClassLibrary  = ALLTRIM( SUBSTR( tcClassLibList, lnBeginPosition ))
				OTHERWISE
					lnBeginPosition = AT( ",", tcClassLibList, lnClassCount-1 ) + 1
					lnEndPosition   = AT( ",", tcClassLibList, lnClassCount ) - 1
					lnLength = (lnEndPosition - lnBeginPosition) + 1
					lcClassLibrary  = ALLTRIM(SUBSTR( tcClassLibList, lnBeginPosition, lnLength ))
				ENDCASE

				lcClassLibrary = STRTRAN( lcClassLibrary, CHR(13), "" )
				lcClassLibrary = STRTRAN( lcClassLibrary, CHR(10), "" )

				IF OCCURS( " ALIAS ", UPPER( lcClassLibrary ) ) > 0
					lcFQClassLibrary = UPPER( lcClassLibrary )
					lnAliasBegin     = AT( " ALIAS ", lcFQClassLibrary )
					lcClassLibrary   = ALLTRIM(LEFT( lcFQClassLibrary, lnAliasBegin ))
					lcAlias          = ALLTRIM(SUBSTR( lcFQClassLibrary, lnAliasBegin + 7 ))
				ELSE
					lcAlias = ""
				ENDIF

				lcFileName = ALLTRIM(UPPER(JUSTFNAME( lcClassLibrary )))

				IF .NOT. ( lcFileName $ lcClassList )
					IF .NOT. EMPTY( lcAlias )
						SET CLASSLIB TO (lcClassLibrary) ALIAS (lcAlias) ADDITIVE
					ELSE
						SET CLASSLIB TO (lcClassLibrary ) ADDITIVE
					ENDIF
				ENDIF

			ENDFOR
		ENDIF

		IF .NOT. ( ALLTRIM(UPPER( tcClassLibrary )) $ lcClassList )

			DO CASE 
			CASE .NOT. EMPTY( tcClassLibrary ) AND .NOT. EMPTY( tcClassFile )
				SET CLASSLIB TO ( tcClassLibrary ) IN ( tcClassFile ) ADDITIVE
			CASE .NOT. EMPTY( tcClassLibrary )
				SET CLASSLIB TO ( tcClassLibrary ) ADDITIVE
			ENDCASE

		ENDIF
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Sets class libraries required to support the
	*)	object being created. The basic purpose of this
	*)	method is to supply the information required
	*)	to run the SET CLASSLIB command.  Note that
	*)	class libraries are not SET if they are already 
	*)	opened.
	*-------------------------------------------------------
	*$ Scope:       PROTECTED
	*$ Parameters:  
	*$	1.	<cClassLibrary> - name of the .VCX or .PRG
	*$		that contains the class definition.
	*$	2.	<cClassFile> - Specifies a Visual FoxPro 
	*$		application file (.app) or executable 
	*$		file (.exe) containing the visual class library.
	*$	3.	<cClassLibList> - comma delimited list of
	*$		class libraries needed to support the object
	*$		being manufactured
	*$ Usage:       
	*$	CAbstractFactory::SetClassLibraries( <cClassLibrary> , ;
	*$                                       <cClassFile>    , ;
	*$                                       <cClassLibList>   )	
	*$ Example:     
	*$	This.SetClassLibraries( lcClassLibrary , ;
	*$	                        lcClassFile    , ;
	*$	                        lcClassLibList   )
	*$ Returns:   LOGICAL .T. by default  
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////


	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  SetProcedures()
	*-------------------------------------------------------
	PROTECTED PROCEDURE SetProcedures
		LPARAMETERS tcProcList

		            
		lcProcList = UPPER(ALLTRIM(SET( 'PROC' )))

		IF .NOT. EMPTY( tcProcList )
			lnTotalProcs = OCCURS( ",", tcProcList ) + 1
			FOR lnProcCount = 1 TO lnTotalProcs
				DO CASE
				CASE lnProcCount   = 1
					lnEndPosition  = AT( ",", tcProcList, 1 ) - 1
					lcProc         = ALLTRIM(LEFT( tcProcList, lnEndPosition ))
				CASE lnProcCount = lnTotalProcs
					lnBeginPosition = AT( ",", tcProcList, lnProcCount-1 ) + 1
					lcProc          = ALLTRIM( SUBSTR( tcProcList, lnBeginPosition ))
				OTHERWISE
					lnBeginPosition = AT( ",", tcProcList, lnProcCount-1 ) + 1
					lnEndPosition   = AT( ",", tcProcList, lnProcCount ) - 1
					lnLength = (lnEndPosition - lnBeginPosition) + 1
					lcProc          = ALLTRIM(SUBSTR( tcProcList, lnBeginPosition, lnLength ))
				ENDCASE

				lcProc = STRTRAN( lcProc, CHR(13), "" )
				lcProc = STRTRAN( lcProc, CHR(10), "" )

				lcFileName = ALLTRIM(UPPER(JUSTFNAME( lcProc )))

				IF .NOT. ( lcFileName $ lcPRocList )
					SET PROC TO (lcPRoc ) ADDITIVE
				ENDIF
			ENDFOR
		ENDIF
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Sets the procedures required to support the
	*)	object being created. The basic purpose of this
	*)	method is to supply the information required
	*)	to run the SET PROC command.  Note that
	*)	procedures are not SET if they are already 
	*)	opened.
	*-------------------------------------------------------
	*$ Scope:       PROTECTED
	*$ Parameters:  
	*$	1.	<cProcList> - comma delimited list of
	*$		procedure files needed to support the object
	*$		being manufactured
	*$ Usage:       
	*$	CAbstractFactory::SetProcedures( <cProcList> )	
	*$ Example:     
	*$	This.SetProcedures( lcProcList )
	*$ Returns:   LOGICAL .T. by default  
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////


	*---------------- Location Section ---------------------
	*} Program: CFactory.PRG
	*} Class:   CAbstractFactory
	*} Method:  InstantiateObject()
	*-------------------------------------------------------
	PROTECTED PROCEDURE InstantiateObject
		LPARAMETER tcClassName

		LOCAL loObject

		loObject = .NULL.

		IF .NOT. EMPTY( tcClassName )AND TYPE( 'tcClassName' ) == "C"
			loObject = CREATEOBJECT( tcClassName )
			IF TYPE( 'loObject' ) != 'O' AND This.lDisplayErrorMessage
				=MsgSvc( 'CFACTORY::CouldNotInstantiateObject', tcClassName )
			ENDIF
		ENDIF

		RETURN loObject
	ENDPROC
	*------------------- Usage Section ---------------------
	*) Description:
	*)	Takes the responsibility of actually creating the
	*)	object using the specified class definition name.
	*-------------------------------------------------------
	*$ Scope:       PROTECTED
	*$ Parameters:  
	*$	1.	<cClassName> - the class definition to use
	*$		to create the desired object.
	*$ Usage:       
	*$	cAbstractFactory::InstantiateObject( <cClassName> )
	*$ Example:     
	*$	THIS.InstantiateObject( lcClassName )
	*$ Returns:  OBJECT
	*$	Reference to the object created by the CREATEOBJ()
	*$	function.
	*----------------- Maintenance Section -----------------
	*^ Change Log:
	*-------------------------------------------------------
	*///////////////////////////////////////////////////////

ENDDEFINE
